# 
# drop.bro
#
#  This verison of drop.bro implements a drop/restore mechanism, 
#   where the first time a system is dropped for scanning, 
#   it is restored 5 minutes after the last connection
#   attempt seen.  If a connection attempt is subsequently seen, the system is
#   blocked again. 
#
# This is necessary when using Bro to update router ACLs when:
#     - the number of ACLs slots in the router is limited
#     - you care about possible reuse of dynamic IP from hostile hosts
#        to non-hostile hosts
#
# original code by Jim Mellander, LBNL
# updated by Brian Tierney, LBNL
#
#

module Drop;

export {
	redef enum Notice += {
		AddressDropped,		# connectivity w/ given address has been dropped
		AddressDropIgnored,	# a request to drop connectivity has been ignored
		AddressRestored,        # connectivity w/ given address has been restored
        	AddressRestoreIgnored,  # a request to restore connectivity has been ignored
        	RepeatAddressDropped,
		RepeatAddressDropIgnored,
	};

	const can_drop_connectivity = F &redef;
	const drop_connectivity_script = "drop-connectivity" &redef;

	const use_catch_release = T &redef;  # usually want to restore dropped hosts
	const restore_connectivity_script = "restore-connectivity" &redef;

	const shut_down_scans: set[port] &redef;
	const shut_down_all_scans = T &redef;
	const shut_down_thresh = 100 &redef; # drop host after this many failed attempts

	const root_servers = {
		a.root-servers.net, b.root-servers.net, c.root-servers.net,
		d.root-servers.net, e.root-servers.net, f.root-servers.net,
		g.root-servers.net, h.root-servers.net, i.root-servers.net,
		j.root-servers.net, k.root-servers.net, l.root-servers.net,
		m.root-servers.net,
	} &redef;

	const gtld_servers = {
		a.gtld-servers.net, b.gtld-servers.net, c.gtld-servers.net,
		d.gtld-servers.net, e.gtld-servers.net, f.gtld-servers.net,
		g.gtld-servers.net, h.gtld-servers.net, i.gtld-servers.net,
		j.gtld-servers.net, k.gtld-servers.net, l.gtld-servers.net,
		m.gtld-servers.net,
	} &redef;

	const never_shut_down = {
		root_servers, gtld_servers,
	} &redef;

	const never_drop_nets: set[subnet] &redef;

	global conn_count: table[addr] of count &default=0 &read_expire = 7 days;

	global drop_address: function(a: addr, msg: string);

	type restore_rec: record {
         	dropped: bool &default=F;    
         	tot_drop_time: interval;
         	restore_time: time;     
         	event_scheduled: bool &default=F;    
 	};
 	global restore_info: table[addr] of restore_rec &read_expire = 1 day ;

	global drop_time = 5 min &redef;  # normal drop interval
	global long_drop_time = 12 hr &redef;  # for repeat offenders
	# If the total drop time timeout reaches this point, the host is 
	#	dropped for long_drop_time 
	const persistant_scanner_time = 2 hr &redef;
}

function init_restore_rec(): restore_rec
{
	local rec: restore_rec;

	rec$dropped = F;
	rec$tot_drop_time = 0 sec;
	rec$restore_time= 0;
	rec$event_scheduled=F;

	return rec;
}

function drop_address(a: addr, msg: string)
{
        msg = fmt("%s (%d successful connection%s)", msg, conn_count[a],
                conn_count[a] == 1 ? "" : "s");

	if ( ! can_drop_connectivity || a in never_shut_down ||
	     a in never_drop_nets )
		NOTICE([$note=AddressDropIgnored, $src=a,
			$msg=fmt("ignoring request to drop %s%s", a, msg)]);
	else
	{
		if (a !in restore_info)
		{
			local rr: restore_rec;
			rr = init_restore_rec();
			restore_info[a] = rr;
		}

		if ( !restore_info[a]$dropped ) # if not already dropped
		{
			event DropEvent( a, msg );  # signal master drop handler to drop this
		        restore_info[a]$dropped = T;

			if ( use_catch_release )
			{
				restore_info[a]$tot_drop_time += drop_time;
				# Schedule the restore of the system
				restore_info[a]$restore_time=network_time()+drop_time;
				schedule drop_time { restore_scanner(a) };
				restore_info[a]$event_scheduled=T;
			}
		}
	}
}



# Keeps count of connections established by IP
# This is used to provide a clue to offline analysis engines whether a drop
# was "interesting" -- if there were no successful connections, it wouldn't be

event connection_established(c: connection)
{
	if ( use_catch_release )
		++conn_count[c$id$orig_h];
}


function repeat_drop_address(a: addr)
{
        local msg = fmt("(%d successful connection%s)", conn_count[a],
                conn_count[a] == 1 ? "" : "s");

	if ( !restore_info[a]$dropped )
	{
		if ( ! can_drop_connectivity || a in never_shut_down ||
				a in never_drop_nets )
                	NOTICE([$note=RepeatAddressDropIgnored, $src=a, $msg=msg]);
		else
			event DropEvent( a, msg );  # signal master drop handler to drop this again

		restore_info[a]$dropped = T;
	}

}


# Keep track of the last time a scanning system connected.
function check_scanner(a: addr)
{
        if ( use_catch_release && a in restore_info && !restore_info[a]$dropped )
        {
                # Check to see if the system should be dropped -
                #  We know that the system was previously dropped, or
                #  we wouldnt have gotten this far!

                repeat_drop_address(a);

                if ( restore_info[a]$tot_drop_time < persistant_scanner_time )
		{
			restore_info[a]$restore_time=network_time()+drop_time;
			restore_info[a]$tot_drop_time += drop_time;
                        schedule drop_time { restore_scanner(a) };
		}
		else
		{
			restore_info[a]$restore_time=network_time()+long_drop_time;
			restore_info[a]$tot_drop_time += long_drop_time;
                        schedule long_drop_time { restore_scanner(a) };
		}
		restore_info[a]$event_scheduled=T;
        }
}


event new_connection(c: connection)
{
# 1 connection from a previously dropped system is
# enough to trigger it to be dropped again.
	check_scanner(c$id$orig_h);
}


# Restore a previously scanning system.
event restore_scanner(a: addr)
{
	if ( a in restore_info && restore_info[a]$dropped )
	{
        	local msg = fmt("(%d successful connection%s) ", 
			conn_count[a], conn_count[a] == 1 ? "" : "s" );

        	if ( ! can_drop_connectivity || a in never_shut_down || a in never_drop_nets )
                	NOTICE([$note=AddressRestoreIgnored, $src=a, $msg=msg]);
        	else
			# signal master drop handler to Restore this host
	       		event RestoreEvent( a, msg );  

		restore_info[a]$event_scheduled=F;
		restore_info[a]$dropped = F;
	}
}

